from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.contrib.auth.forms import (
    AuthenticationForm as BaseAuthenticationForm,
    PasswordResetForm as BasePasswordResetForm,
    SetPasswordForm as BaseSetPasswordForm)

from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import Layout, Row, Column, Submit, HTML

User = get_user_model()


class RegistrationForm(forms.Form):
    """
    Copied from registration.forms.RegistrationForm but
    strip off username and added unique email validation.
    """
    required_css_class = 'required'

    name = forms.CharField(label=_("Name"))
    email = forms.EmailField(label=_("E-mail"))
    password1 = forms.CharField(widget=forms.PasswordInput,
                                label=_("Password"))
    password2 = forms.CharField(widget=forms.PasswordInput,
                                label=_("Password (again)"))

    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.layout = Layout(
            Row(Column('name')),
            Row(Column('email')),
            Row(Column('password1')),
            Row(Column('password2')),
            Row(Column(Submit('submit', _('Submit'))),
                css_class='form-actions'),
        )
        super(RegistrationForm, self).__init__(*args, **kwargs)

    def clean_email(self):
        """
        Validate that the supplied email address is unique for the
        site.
        """
        if User.objects.filter(email__iexact=self.cleaned_data['email']):
            raise forms.ValidationError(_("This email is already taken."))
        return self.cleaned_data['email']

    def clean(self):
        """
        Verifiy that the values entered into the two password fields
        match. Note that an error here will end up in
        ``non_field_errors()`` because it doesn't apply to a single
        field.
        """
        if 'password1' in self.cleaned_data and 'password2' in self.cleaned_data:
            if self.cleaned_data['password1'] != self.cleaned_data['password2']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return self.cleaned_data


class AuthenticationForm(BaseAuthenticationForm):
    def __init__(self, *args, **kwargs):
        reset_url = reverse('auth_password_reset')
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.layout = Layout(
            Row(Column('username')),
            Row(Column('password')),
            Row(Column(Submit('submit', _('Login')),
                       css_class='large-2'),
                Column(HTML('<a href="%s" class="forgot-password">Forgot password?</a>' % reset_url),
                       css_class='large-10'),
                css_class='form-actions collapse'),
        )
        super(AuthenticationForm, self).__init__(*args, **kwargs)


class PasswordResetForm(BasePasswordResetForm):
    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.layout = Layout(
            Row(Column('email')),
            Row(Column(Submit('submit', _('Submit'))),
                css_class='form-actions'),
        )
        super(PasswordResetForm, self).__init__(*args, **kwargs)


class SetPasswordForm(BaseSetPasswordForm):
    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_method = 'post'
        self.helper.layout = Layout(
            Row(Column('new_password1')),
            Row(Column('new_password2')),
            Row(Column(Submit('submit', _('Update'))),
                css_class='form-actions'),
        )
        super(SetPasswordForm, self).__init__(*args, **kwargs)



from django.contrib.sites.models import RequestSite
from django.contrib.sites.models import Site

from registration import signals
from registration.models import RegistrationProfile
from registration.backends.default.views import (
    RegistrationView as BaseRegistrationView)

from .forms import RegistrationForm


class RegistrationView(BaseRegistrationView):
    form_class = RegistrationForm

    def register(self, request, **cleaned_data):
        """ We override register since the old one still uses username. """
        name, email, password = (cleaned_data['name'],
                                 cleaned_data['email'],
                                 cleaned_data['password1'])
        if Site._meta.installed:
            site = Site.objects.get_current()
        else:
            site = RequestSite(request)
        new_user = RegistrationProfile.objects.create_inactive_user(
            name, email, password, site)
        signals.user_registered.send(sender=self.__class__,
                                     user=new_user,
                                     request=request)
        return new_user



"""
An override to registration.backend.default.urls so we
can pass our custom RegistrationView with a custom form.
"""

from django.conf.urls import url

from registration.backends.default.urls import urlpatterns
from registration.auth_urls import urlpatterns as auth_urlpatterns

from .views import RegistrationView
from .forms import AuthenticationForm, PasswordResetForm, SetPasswordForm


# override the url for accounts/register
# with our custom RegistrationView.
urlpatterns[2] = url(
    r'^register/$',
    RegistrationView.as_view(),
    name='registration_register'
)

# remove relative resolution of auth_urls
# and instead concatenate auth_urls directly.
del urlpatterns[5]
urlpatterns += auth_urlpatterns

# override forms
urlpatterns[5].default_args.update({'authentication_form': AuthenticationForm})
urlpatterns[9].default_args.update({'password_reset_form': PasswordResetForm})
urlpatterns[10].default_args.update({'set_password_form': SetPasswordForm})



from django import forms
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.forms import ReadOnlyPasswordHashField
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse

User = get_user_model()


class UserCreationForm(forms.ModelForm):
    """
    A form for creating new users. Includes all the
    required fields, plus a repeated password.
    """
    password1 = forms.CharField(label="Password", widget=forms.PasswordInput)
    password2 = forms.CharField(label="Password confirmation",
                                widget=forms.PasswordInput)

    class Meta:
        model = User
        fields = ("name", "email")

    def clean_password2(self):
        # Check that the two password entries match
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError("Passwords don't match")
        return password2

    def save(self, commit=True):
        # Save the provided password in hashed format
        user = super(UserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class UserChangeForm(forms.ModelForm):
    """
    A form for updating users. Includes all the fields
    on the user, but replaces the password field with
    admin's password hash display field.
    """
    password = ReadOnlyPasswordHashField(
        help_text=_("Raw passwords are not stored, so there is no way to see "
                    "this user's password, but you can change the password "
                    "using <a href=\"password/\">this form</a>."))

    class Meta:
        model = User

    def clean_password(self):
        # Regardless of what the user provides, return the
        # initial value. This is done here, rather than on
        # the field, because the field does not have access
        # to the initial value
        return self.initial["password"]


class UserAdmin(UserAdmin):
    # Set the add/modify forms
    add_form = UserCreationForm
    form = UserChangeForm

    # The fields to be used in displaying the User model.
    # These override the definitions on the base UserAdmin
    # that reference specific fields on auth.User.
    list_display = ("name", "email", "is_staff", "impersonate_link")
    list_filter = ("is_staff", "is_superuser", "is_active", "groups")
    search_fields = ("name", "email")
    ordering = ("name",)
    filter_horizontal = ("groups", "user_permissions",)
    fieldsets = (
        (None, {"fields": ("email", "password")}),
        ("Personal info", {"fields": ("name",)}),
        ("Permissions", {"fields": ("is_active",
                                    "is_staff",
                                    "is_superuser",
                                    "groups",
                                    "user_permissions")}),
        ("Important dates", {"fields": ("last_login",)}),
    )
    add_fieldsets = (
        (None, {"classes": ("wide",),
                "fields": ("name",
                           "email",
                           "password1",
                           "password2")}),
    )

    def impersonate_link(self, obj):
        return u'<a href="%s">Impersonate</a>' % reverse("impersonate-start",
                                                         args=(obj.id,))
    impersonate_link.allow_tags = True
    impersonate_link.short_description = "impersonate"


# Register the new UserAdmin
admin.site.register(User, UserAdmin)



"""
Patches for the registration.models.RegistrationManager methods.
"""

import hashlib
import random
import warnings
from autoslug import AutoSlugField

from django.db import transaction
from django.utils import timezone
from django.contrib.gis.db import models
from django.core.mail import send_mail
from django.utils.http import urlquote
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ImproperlyConfigured
from django.contrib.auth.models import (BaseUserManager,
                                        AbstractBaseUser,
                                        PermissionsMixin,
                                        SiteProfileNotAvailable)


class UserManager(BaseUserManager):
    def create_user(self, name, email, password=None):
        """
        Creates and saves a User with the given email, name and password.
        """
        if not name:
            msg = "Users must have a name"
            raise ValueError(msg)

        if not email:
            msg = "Users must have an email address"
            raise ValueError(msg)

        user = self.model(name=name, email=UserManager.normalize_email(email))
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, name, email, password):
        """
        Creates and saves a superuser with the given email, name and password.
        """
        user = self.create_user(name=name, email=email, password=password)
        user.is_admin = True
        user.is_staff = True
        user.is_superuser = True
        user.save(using=self._db)
        return user


class User(AbstractBaseUser, PermissionsMixin):
    name = models.CharField(max_length=255)
    email = models.EmailField(verbose_name=u'email address',
                              max_length=255,
                              unique=True,
                              db_index=True)

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = ["name"]

    is_active = models.BooleanField(default=True)
    is_admin = models.BooleanField(default=False)
    is_staff = models.BooleanField(default=False)
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    slug = AutoSlugField(populate_from='name', unique=True)

    objects = UserManager()

    def get_absolute_url(self):
        return "/users/%s/" % urlquote(self.slug)

    def get_full_name(self):
        return self.name

    def get_short_name(self):
        return self.email

    def email_user(self, subject, message, from_email=None):
        send_mail(subject, message, from_email, [self.email])

    def get_profile(self):
        warnings.warn("The use of AUTH_PROFILE_MODULE to define user profiles has been deprecated.",
            PendingDeprecationWarning)
        if not hasattr(self, '_profile_cache'):
            from django.conf import settings
            if not getattr(settings, 'AUTH_PROFILE_MODULE', False):
                raise SiteProfileNotAvailable(
                    'You need to set AUTH_PROFILE_MODULE in your project '
                    'settings')
            try:
                app_label, model_name = settings.AUTH_PROFILE_MODULE.split('.')
            except ValueError:
                raise SiteProfileNotAvailable(
                    'app_label and model_name should be separated by a dot in '
                    'the AUTH_PROFILE_MODULE setting')
            try:
                model = models.get_model(app_label, model_name)
                if model is None:
                    raise SiteProfileNotAvailable(
                        'Unable to load the profile model, check '
                        'AUTH_PROFILE_MODULE in your project settings')
                self._profile_cache = model._default_manager.using(
                    self._state.db).get(user__id__exact=self.id)
                self._profile_cache.user = self
            except (ImportError, ImproperlyConfigured):
                raise SiteProfileNotAvailable
        return self._profile_cache

    def __unicode__(self):
        return self.email


@transaction.commit_on_success
def create_inactive_user(self, name, email, password, site, send_email=True):
    new_user = User.objects.create_user(name, email, password)
    new_user.is_active = False
    new_user.save()

    registration_profile = self.create_profile(new_user)

    if send_email:
        registration_profile.send_activation_email(site)

    return new_user


def create_profile(self, user):
    salt = hashlib.sha1(str(random.random())).hexdigest()[:5]
    activation_key = hashlib.sha1(salt+user.email).hexdigest()
    return self.create(user=user, activation_key=activation_key)



from .models import create_inactive_user, create_profile


def patch():
    """ Apply patch for django-registration's manager methods. """
    from registration import models
    setattr(models.RegistrationManager,
            'create_inactive_user',
            create_inactive_user)
    setattr(models.RegistrationManager,
            'create_profile',
            create_profile)


patch()



#!/usr/bin/env python

import os
import sys
import usernameless

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist register upload')
    sys.exit()

packages = ['usernameless']
requires = ['django-impersonate==0.8.0',
            'django-registration==1.0',
            'django-autoslug==1.7.1']

setup(
    name='django-usernameless',
    version=usernameless.__version__,
    packages=packages,
    description='Custom user without username for Django-1.5',
    long_description=open('README.md').read() + '\n\n' +
                     open('HISTORY.rst').read(),
    author='Marconi Moreto',
    author_email='caketoad@gmail.com',
    url='https://github.com/marconi/django-usernameless',
    zip_safe=False,
    install_requires=requires,
)
